// deferredHairNoNM.fx
// by Julian "Jibb" Smart
// 2009

/*
Colours in target 0 and 3 (ambient and lighting colour, respectively),
Depth in target 1,
Normals in target 2
*/

const float4x4 matWorldViewProj;
const float4x4 matWorld;
const float4 vecViewDir;
const float4 vecViewPos;
const float4 vecSunDir;

// view's y and z
float4 vecSkill1;
float4 vecSkill5;

float toneMappingFactor_var;
float sunFactor_var;

float3x3 matTangent;

texture entSkin1;
sampler ColourSampler = sampler_state {
	Texture = <entSkin1>;
};

texture entSkin2;
sampler SpecSampler = sampler_state {
	Texture = <entSkin2>;
};

void VS	(in float4 inPos			:	POSITION,
			in float3 inNormal		:	NORMAL,
			in float2 inTex			:	TEXCOORD0,
			in float4 inTangent		:	TEXCOORD2,
			out float4 outPos			:	POSITION,
			out float2 outTex			:	TEXCOORD0,
			out float3 outTan			:	TEXCOORD1,
			out float3 outColour		:	TEXCOORD2,
			out float4 screenPos		:	TEXCOORD3,
			out float3 viewDiff		:	TEXCOORD4,
			out float3 outNormal		:	TEXCOORD5) {
	outPos = mul(inPos, matWorldViewProj);
	screenPos = outPos;
	outTex = inTex;
	// tangent vector
	outTan = mul(cross(inTangent.xyz, inNormal), matWorld);
	// view->position vector
	viewDiff = mul(inPos.xyz, matWorld);
	viewDiff -= vecViewPos.xyz;
	// normals
	outNormal = mul(inNormal, matWorld);
	outColour.r = -dot(vecSkill1.xyz, outNormal);
	outColour.g = dot(vecSkill5.xyz, outNormal);
	outColour.b = dot(vecViewDir, outNormal);
	outColour = outColour*0.5 + 0.5;
}

float4 PS	(in float2 inTex		:	TEXCOORD0,
				in float3 inTan		:	TEXCOORD1,
				in float4 inColour	:	TEXCOORD2,
				in float4 inPos		:	TEXCOORD3,
				in float3 viewDiff	:	TEXCOORD4,
				in float3 inNormal	:	TEXCOORD5,
				out float4 depth		:	COLOR1,
				out float4 normals	:	COLOR2,
				out float4 colours	:	COLOR3) : COLOR0
{
	// colours
	colours = tex2D(ColourSampler, inTex);
	// forward-rendered light
	float4 specFactors = tex2D(SpecSampler, inTex);
	float alpha = colours.a;
	float shine = abs(acos(dot(inTan, normalize(viewDiff))) - acos(dot(-inTan, vecSunDir)));
	// the shine is the sharpness of the high-light
	shine = pow(saturate(1.0 - (0.318309886 * shine)), specFactors.g * 128) * specFactors.r * 5;
//	shine = pow(1.0 - (0.318309886 * shine), 20) * 3;
	float light = saturate(dot(vecSunDir, -inNormal)) * alpha;
//	light = 1;
	// depth
	depth = float4(inPos.z, 0.0, 0.0, saturate(alpha * 5));
	// normals
	normals.rgb = inColour.rgb;
	normals.a = alpha;
	light *= sunFactor_var;
	float4 ambience;
	ambience.rgb = (colours.rgb * 0.5 + shine * light + light * colours.rgb) * toneMappingFactor_var;
	ambience.a = alpha;
	return ambience;
}

technique NMTech {
	pass p0 {
		AlphaBlendEnable = TRUE;
		VertexShader = compile vs_2_0 VS();
		PixelShader = compile ps_2_0 PS();
	}
}
