// deferredNoNM.fx
// by Julian "Jibb" Smart
// 2009

/*
Colours in target 0 and 3 (ambient and lighting colour, respectively),
Depth in target 1,
Normals in target 2
*/

const float4x4 matWorldViewProj;
const float4x4 matWorld;
const float4 vecViewDir;
const float4 vecSunDir;

// view's y and z
float4 vecSkill1;
float4 vecSkill5;

float toneMappingFactor_var;
float sunFactor_var;

texture entSkin1;
sampler ColourSampler = sampler_state {
	Texture = <entSkin1>;
};

void VS	(in float4 inPos			:	POSITION,
			in float3 inNormal		:	NORMAL,
			in float2 inTex			:	TEXCOORD0,
			out float4 outPos			:	POSITION,
			out float2 outTex			:	TEXCOORD0,
			out float3 outColour		:	TEXCOORD1,
			out float4 outPos2		:	TEXCOORD2,
			out float3 outNormal		:	TEXCOORD3) {
	outPos = mul(inPos, matWorldViewProj);
	outTex = inTex;
	outPos2 = outPos;
	outNormal = mul(inNormal, matWorld);
	outColour.r = -dot(vecSkill1.xyz, outNormal);
	outColour.g = dot(vecSkill5.xyz, outNormal);
	outColour.b = dot(vecViewDir, outNormal);
	outColour = outColour*0.5 + 0.5;
}

float4 PS	(in float2 inTex		:	TEXCOORD0,
				in float3 inColour	:	TEXCOORD1,
				in float4 inPos		:	TEXCOORD2,
				in float3 inNormal	:	TEXCOORD3,
				out float4 depth		:	COLOR1,
				out float4 normals	:	COLOR2,
				out float4 colours	:	COLOR3) : COLOR0
{
	// colours
	colours = tex2D(ColourSampler, inTex);
	float4 ambience = colours;
	ambience.rgb *= 0.5;
	float light = saturate(dot(vecSunDir, -inNormal));
	// depth
	depth = float4(inPos.z, 0.0, 0.0, 1.0);
	// normals
	normals.rgb = inColour.rgb;
	normals.a = 1.0;
//	colours.rgb = max(colours.r, max(colours.g, colours.b));
	return (ambience + light * colours * sunFactor_var) * toneMappingFactor_var;
}

technique DiffuseTech {
	pass p0 {
		VertexShader = compile vs_2_0 VS();
		PixelShader = compile ps_2_0 PS();
	}
}
