// forwardEM.fx
// by Julian "Jibb" Smart
// 2009

const float4x4 matWorldViewProj;
const float4x4 matWorld;
const float4 vecViewDir;
const float4 vecViewPos;
const float4 vecSunDir;

// view's y and z
float4 vecSkill1;
float4 vecSkill5;

float toneMappingFactor_var;
float sunFactor_var;

texture reflection_bmap;
sampler ReflectSampler = sampler_state {
	Texture = <reflection_bmap>;
};

texture entSkin1;
sampler ColourSampler = sampler_state {
	Texture = <entSkin1>;
};

void VS	(in float4 inPos			:	POSITION,
			in float3 inNormal		:	NORMAL,
			in float2 inTex			:	TEXCOORD0,
			in float4 inTangent		:	TEXCOORD2,
			out float4 outPos			:	POSITION,
			out float2 outTex			:	TEXCOORD0,
			out float3 outNormal		:	TEXCOORD1,
			out float3 worldPos		:	TEXCOORD2) {
	outPos = mul(inPos, matWorldViewProj);
	outTex = inTex;
	worldPos = normalize(mul(inPos, matWorld) - vecViewPos.xyz);
	outNormal = mul(inNormal, matWorld);
}

float4 PS	(in float2 inTex		:	TEXCOORD0,
				in float3 inNormal	:	TEXCOORD1,
				in float3 worldPos	:	TEXCOORD2) : COLOR0
{
	// colours
	float4 colours = tex2D(ColourSampler, inTex);
	float light = saturate(dot(vecSunDir, -inNormal));
	// calculate reflection vector
	inNormal = normalize(inNormal);
	float4 mat = texCUBE(ReflectSampler, reflect(worldPos, inNormal));
	mat.rgb = (pow(mat.rgb, 5) + colours.rgb * (light * sunFactor_var + 0.5)) * toneMappingFactor_var;
	mat.a = 1.0;
	return mat;
}

technique NMTech {
	pass p0 {
		AlphaBlendEnable = FALSE;
		ZWriteEnable = TRUE;
		VertexShader = compile vs_2_0 VS();
		PixelShader = compile ps_2_0 PS();
	}
}
