// forwardWithNM.fx
// by Julian "Jibb" Smart
// 2009

const float4x4 matWorldViewProj;
const float4x4 matWorld;
const float4 vecViewDir;
const float4 vecViewPos;
const float4 vecSunDir;

// view's y and z
float4 vecSkill1;
float4 vecSkill5;

float toneMappingFactor_var;
float sunFactor_var;

float3x3 matTangent;

texture entSkin1;
sampler ColourSampler = sampler_state {
	Texture = <entSkin1>;
};

texture entSkin2;
sampler NormalSampler = sampler_state {
	Texture = <entSkin2>;
};

void VS	(in float4 inPos			:	POSITION,
			in float3 inNormal		:	NORMAL,
			in float2 inTex			:	TEXCOORD0,
			in float4 inTangent		:	TEXCOORD2,
			out float4 outPos			:	POSITION,
			out float2 outTex			:	TEXCOORD0,
			out float3 tanSun			:	TEXCOORD1,
			out float3 relPos			:	TEXCOORD2) {
	outPos = mul(inPos, matWorldViewProj);
	outTex = inTex;
	// 3x3 matrix for world space to tangent space
	matTangent[0] = mul(inTangent.xyz, matWorld);
	matTangent[1] = mul(cross(inTangent.xyz, inNormal)*inTangent.w, matWorld);
	matTangent[2] = mul(inNormal, matWorld);
	// sun in tangent space
	tanSun = normalize(mul(matTangent, vecSunDir.xyz));
	// relative position from the view
	relPos = normalize(mul(matTangent, mul(inPos.xyz, matWorld) - vecViewPos.xyz));
}

float4 PS	(in float2 inTex		:	TEXCOORD0,
				in float3 tanSun		:	TEXCOORD1,
				in float3 relPos		:	TEXCOORD2) : COLOR0
{
	// colours
	float4 colours = tex2D(ColourSampler, inTex);
	float4 ambience = colours;
	ambience.rgb *= 0.5;
	// light and shine
/*
	This shader is for normal maps as blender produces them.
	This means that the Blue channel only represents positive
	values of Z, and thus does not need to be unpacked as the
	Red and Green channel need to.
	Also, the Red channel is the wrong way around. Isn't that
	so lame?
*/	
	float4 bumpNormal = tex2D(NormalSampler, inTex);
	bumpNormal.rg = bumpNormal.rg*2 - 1;
	bumpNormal.r *= -1;
	float light = saturate(dot(tanSun, -bumpNormal.rgb));
	float shine = pow(saturate(dot(reflect(tanSun, -bumpNormal.rgb), -relPos)), bumpNormal.a * 25.5) * bumpNormal.a;
	ambience.rgb = (ambience.rgb + (shine + light * colours.rgb) * sunFactor_var) * toneMappingFactor_var;
	return ambience;
}

technique NMTech {
	pass p0 {
		AlphaBlendEnable = FALSE;
		ZWriteEnable = TRUE;
		VertexShader = compile vs_2_0 VS();
		PixelShader = compile ps_2_0 PS();
	}
}
