// softFlat.fx
// Julian "Jibb" Smart
// 2009

const float4x4 matWorldViewProj;

// Softness
float4 vecSkill1;

float toneMappingFactor_var;

float fAlpha;
// Convert clip space to texture space
static const float4x4 mat2Tex = {
	0.5f, 0.f, 0.f, 0.f,
	0.f, -0.5f, 0.f, 0.f,
	0.f, 0.f, 1.0f, 0.f,
	0.5004f, 0.5004f, 0.f, 1.0f
};

// Actual Texture
texture entSkin1;
sampler ColourMapSampler = sampler_state {
	Texture = <entSkin1>;
	MipFilter = Linear;
};

// Depth Map
texture depthMap_bmap;
sampler DepthSampler = sampler_state {
	Texture = <depthMap_bmap>;
	MipFilter = Linear;
};

// VS
void VS	(in float4 inPos		:	POSITION,
			in float2 inTex		:	TEXCOORD0,
			out float4 outPos		:	POSITION,
			out float2 outTex		:	TEXCOORD0,
			out float4 depthTex	:	TEXCOORD1,
			out float4 psPos		:	TEXCOORD2) {
	float4x4 matOffset = mul(matWorldViewProj, mat2Tex);
	outPos = mul(inPos, matWorldViewProj);
	psPos = outPos;
	// Convert clip space to texture space
	depthTex = mul(inPos, matOffset);

	outTex = inTex;
}

float4 PS	(in float2 inTex		:	TEXCOORD0,
				in float4 depthTex	:	TEXCOORD1,
				in float4 inPos		:	TEXCOORD2) : COLOR0 {
	float4 colour = tex2D(ColourMapSampler, inTex);
	// Get this pixel's depth relative to the depth map and computer softness
	float softFactor = saturate(vecSkill1.x * (tex2Dproj(DepthSampler, depthTex).r - inPos.z));
	colour.rgb *= toneMappingFactor_var;
	colour.a *= fAlpha*softFactor*saturate(inPos.z*0.1);
	return colour;
}

technique softParticles {
	pass p0 {
		AlphaBlendEnable = TRUE;
		ZWriteEnable = FALSE;
		ZFunc = ALWAYS;
		DestBlend = INVSRCALPHA;
		SrcBlend = SRCALPHA;
		VertexShader = compile vs_2_0 VS();
		PixelShader = compile ps_2_0 PS();
	}
}
